-----------------------------------------------------------------------------------------------
-- Client Lua Script for LetsGamble
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "GameLib"
require "Apollo"
require "GroupLib"
require "ChatSystemLib"
 
-----------------------------------------------------------------------------------------------
-- LetsGamble Module Definition
-----------------------------------------------------------------------------------------------
local LetsGamble = {} 

local Modes = { "Closest 50", "Lowest", "Highest", "Wild", "Hidden" }
local tt = {
	"The closest player to 50 wins",
	"The player withe the lowest roll wins",
	"The player withe the highest roll wins",
	"The player closest to a random number wins",
	"The lucky number will be displayed after everyone had rolled"
	}
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function LetsGamble:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function LetsGamble:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- LetsGamble OnLoad
-----------------------------------------------------------------------------------------------
function LetsGamble:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("LetsGamble.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- LetsGamble OnDocLoaded
-----------------------------------------------------------------------------------------------
function LetsGamble:OnSave(eType)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		return {
			Offset = self.wndOffset
		}
	end
	return nil
end

function LetsGamble:OnRestore(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character  then
		
		if tSavedData.Offset ~= nil then
			self.wndOffset = tSavedData.Offset
		end
	end
end

function LetsGamble:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "LetsGambleForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)
		--Slash
		Apollo.RegisterSlashCommand("gamble","OnGamble",self)
		
		--Events
		
		--Var
		self.closeTimer = ApolloTimer.Create(20, false, "OnCloseRound", self)
		self.closeTimer:Stop()
		self.reminderTimer = ApolloTimer.Create(15, false, "OnReminder", self)
		self.reminderTimer:Stop()
		self.RoundEnd = ApolloTimer.Create(35, false, "OnEndRound", self)
		self.RoundEnd:Stop()
		
		self.NotClosed = false
		self.Roll = false
		self.Player = {}
		self.active = false
		
		if self.wndOffSet == nil then
			self.wndOffSet = {62,46,437,225}
		end
		-- Do additional Addon initialization here
		self:SetWindows()
		self:LoadModes()
		self:LoadSlider()
	end
end

-----------------------------------------------------------------------------------------------
-- LetsGamble Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
function LetsGamble:OnGamble(Command, Message)
	if Message == nil or Message == "" then
		self:Show()
	else
		local mode, bet = self:Split(Message, " ")
		if bet == nil or tonumber(bet) == nil then 
			bet = 1
		else
			bet = tonumber(bet)
		end
		self.wndMain:FindChild("EditBox"):SetText(tostring(bet))
		self.wndMain:FindChild("SliderBar"):SetValue(bet)
		if mode == "50" then
			self.Mode = "Closest 50"
			self:StartNewRound()
		elseif mode == "low" or mode == "Low" or mode == "Lowest" or mode == "lowest" then
			self.Mode = "Lowest"
			self:StartNewRound()
		elseif mode == "high" or mode == "High" or mode == "Highest" or mode == "highest" then
			self.Mode = "Highest"
			self:StartNewRound()
		elseif mode == "wild" or mode == "Wild" then
			self.Mode = "Wild"
			self:StartNewRound()
		elseif mode == "hidden" or mode == "Hidden" then
			self.Mode = "Hidden"
			self:StartNewRound()
		else
			ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_System , "Mode not found" , "Idiot" )
		end
	end
end

-----------------------------------------------------------------------------------------------
-- LetsGambleForm Functions
-----------------------------------------------------------------------------------------------
function LetsGamble:NewMessage(channelSource, tMessageInfo)
	if not self.NotClosed and not self.Roll then return end
	if not GroupLib.InGroup() and not GroupLib.InRaid() then return end
	local Sender = tMessageInfo["strSender"]
	local text = tMessageInfo["arMessageSegments"][1]["strText"]
	if self.Player == nil then 
		self.Player = {}
	end
	if self.NotClosed then
		local tChannel = ChatSystemLib.ChatChannel_Party
		tChannel = ChatSystemLib.GetChannels()[tChannel]
		if channelSource == tChannel then
			if text == "+" then
				self.Player[Sender] = 0
			end
		end
	elseif self.Roll then
		local tChannel = ChatSystemLib.ChatChannel_System
		tChannel = ChatSystemLib.GetChannels()[tChannel]
		if channelSource ==  tChannel then
			local a,b,c,d,e = self:Split(text, " ")
			if d == nil or e == nil or c == nil or b == nil or a == nil then return end 
			Sender = a.." "..b
			if self.Player[Sender] ~= nil and self.Player[Sender] == 0 then
				if e == "(1-100)" then
					self.Player[Sender] = d
				end
			end
		end
	end
end	

function LetsGamble:SetWindows()
	local l,t,r,b = unpack(self.wndOffSet)
	self.wndMain:SetAnchorOffsets(l,t,r,b)
end

function LetsGamble:LoadSlider()
	local slider = self.wndMain:FindChild("SliderBar")
	local fMin,fMax, fStep = 1,100,1
	slider:SetMinMax(fMin, fMax, fStep)
	slider:SetValue(1)
	self.wndMain:FindChild("Scale_Bet:EditBox"):SetText("1")
end

function LetsGamble:LoadModes()
	local ModeGrid = self.wndMain:FindChild("Modes")
	for i,j in pairs(Modes) do
		local newMode = Apollo.LoadForm(self.xmlDoc, "Mode", ModeGrid, self)
		newMode:SetText(j)
	end
	ModeGrid:ArrangeChildrenVert(0)
end

function LetsGamble:Show()
	if not self.wndMain:IsShown() then
		self.wndMain:Show(true)
		self.wndMain:ToFront()
	else
		self.wndMain:Show(false)
	end
end

function LetsGamble:StartNewRound()
	if self.active then return end
	if self.Mode == nil then
		ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_System , "Select Mode" , "Idiot" )
		return
	end
	Apollo.RegisterEventHandler("ChatMessage", "NewMessage",self)
	if not GroupLib.InGroup() and not GroupLib.InRaid() then
		return
	end
	local tChannel = ChatSystemLib.ChatChannel_Party
	local strPrefix    = "Let's gamble: "
	
	--Check Editbox if its a number
	if tonumber(self.wndMain:FindChild("EditBox"):GetText()) ~= nil then
		self.wndMain:FindChild("SliderBar"):SetValue(tonumber(self.wndMain:FindChild("EditBox"):GetText()))
	else
		ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_System , "Commitment is not a number" , "Idiot" )
		return
	end
	
	local strBet      = "Bet: "..self.wndMain:FindChild("EditBox"):GetText().." Platin"
	tChannel = ChatSystemLib.GetChannels()[tChannel]
	
	self.hidenNumber = math.random(100)
	local strAscending = ""
	if self.Mode == "Wild" then
		strAscending = " | Mode: "..self.Mode.." ("..tostring(self.hidenNumber)..") (join with +)"
	else
		strAscending = " | Mode: "..self.Mode.." (join with +)"
	end
	
	self.closeTimer:Start()
	self.reminderTimer:Start()
	self.RoundEnd:Start()
	self.NotClosed = true
	self.active = true

	-- Push the accepted format for a begin roll to the party channel.
	tChannel:Send(strPrefix .. strBet .. strAscending)
end

function LetsGamble:OnJoin()
	self.joinTimer:Stop()
	if not GroupLib.InGroup() and not GroupLib.InRaid() then
		return
	end
	local tChannel = ChatSystemLib.ChatChannel_Party
	local strPrefix    = "+"
	tChannel = ChatSystemLib.GetChannels()[tChannel]
	tChannel:Send(strPrefix)
end

function LetsGamble:OnReminder()
	if not GroupLib.InGroup() and not GroupLib.InRaid() then
		return
	end
	local tChannel = ChatSystemLib.ChatChannel_Party
	tChannel = ChatSystemLib.GetChannels()[tChannel]
	local strPrefix    = "Close in 5..."
	tChannel:Send(strPrefix)
end

function LetsGamble:OnCloseRound()
	self.reminderTimer:Stop()
	
	if not GroupLib.InGroup() and not GroupLib.InRaid() then
		return
	end
	local tChannel = ChatSystemLib.ChatChannel_Party
	tChannel = ChatSystemLib.GetChannels()[tChannel]
	self.NotClosed = false
	self.Roll = true
	local strPrefix    = "---Start Rolls ---"
	tChannel:Send(strPrefix)
end

function LetsGamble:OnEndRound()
	self.RoundEnd:Stop()
	self.Roll = false
	if not GroupLib.InGroup() and not GroupLib.InRaid() then
		return
	end
	local tChannel = ChatSystemLib.ChatChannel_Party
	tChannel = ChatSystemLib.GetChannels()[tChannel]
	local strPrefix    = "---Round Ended---"
	tChannel:Send(strPrefix)
	if self.Player == nil then return end
	local Winner = {}
		local max = 0
		
	if self.Mode == "Closest 50" then
		max = 301
		for i,j in pairs(self.Player) do
			if math.abs(tonumber(j)-50) < math.abs(max-50) then
				max = tonumber(j)
			end
		end
	elseif self.Mode == "Lowest" then
		max = 301
		for i,j in pairs(self.Player) do
			if tonumber(j) < max then
				max = tonumber(j)
			end
		end	
	elseif self.Mode == "Highest" then
		for i,j in pairs(self.Player) do
			if tonumber(j) > max then
				max = tonumber(j)
			end
		end	
	elseif self.Mode == "Wild" then
		max = 301
		for i,j in pairs(self.Player) do
			if math.abs(tonumber(j)-self.hidenNumber) < math.abs(max-self.hidenNumber) then
				max = tonumber(j)
			end
		end	
	elseif self.Mode == "Hidden" then
		max = 301
		for i,j in pairs(self.Player) do
			if math.abs(tonumber(j)-self.hidenNumber) < math.abs(max-self.hidenNumber) then
				max = tonumber(j)
			end
		end
	end
	if self.Mode == "Hidden" then
		tChannel:Send("The number was: "..self.hidenNumber)
	end
	for i,j in pairs(self.Player) do
		if max == tonumber(j) then
			table.insert(Winner,i)
		end
	end
	local str = ""
	if #Winner > 1 then
		for i = 1,(#Winner-1) do
			str = str..Winner[i].." | "
		end
		str = str..Winner[#Winner]
	else
		str = Winner[1]
	end
	if str == nil or str == "" then
		tChannel:Send("no Winner :(")
	else
		tChannel:Send("Winner: "..str)
	end
	self.Player = nil
	Apollo.RemoveEventHandler("ChatMessage", self)
	self.active = false
end

function LetsGamble:OnRoll()
	self.rollTimer:Stop()
	ChatSystemLib.Command("/roll")
end

function LetsGamble:OnWindowMoved( wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom )
	local l,t,r,b = wndControl:GetAnchorOffsets()
	self.wndSets = {l,t,r,b}
end

--[[if (tMessagePost.eChannelType == ChatSystemLib.ChatChannel_System and self.bIsRolling) then
			self:OnChatParseTimer_SystemParse(tMessagePost)
		elseif tMessagePost.eChannelType == self.eParseChannel then
			self:OnChatParseTimer_PartyMessage(tMessagePost)
		end]]

function LetsGamble:OnModeClick( wndHandler, wndControl, eMouseButton )
	self.Mode = wndHandler:GetText()
	
	for i,k in pairs(self.wndMain:FindChild("Modes"):GetChildren()) do
		k:FindChild("Highlight"):Show(false)
		k:SetTooltip(tt[i])
	end
	wndHandler:FindChild("Highlight"):Show(true)
end


function LetsGamble:OnEditboxEscape( wndHandler, wndControl )
	wndHandler:SetText(tostring(self.wndMain:FindChild("SliderBar"):GetValue()))
end

-- Compatibility: Lua-5.1 thx to Johan Lindström (Jabbit-EU, Joxye Nadrax / Wildstar) for this function
function LetsGamble:Split(str, pat)
   local t = {}  -- NOTE: use {n = 0} in Lua-5.0
   local fpat = "(.-)" .. pat
   local last_end = 1
   local s, e, cap = str:find(fpat, 1)
   while s do
      if s ~= 1 or cap ~= "" then
	 table.insert(t,cap)
      end
      last_end = e+1
      s, e, cap = str:find(fpat, last_end)
   end
   if last_end <= #str then
      cap = str:sub(last_end)
      table.insert(t, cap)
   end
   return unpack(t)
end

function LetsGamble:OnBetSlideChanged( wndHandler, wndControl, fNewValue, fOldValue )
	wndHandler:GetParent():FindChild("EditBox"):SetText(tostring(math.floor(fNewValue)))
	wndControl:SetValue(math.floor(fNewValue))
end


-----------------------------------------------------------------------------------------------
-- LetsGamble Instance
-----------------------------------------------------------------------------------------------
local LetsGambleInst = LetsGamble:new()
LetsGambleInst:Init()
